// src/index.tsx
import * as React from "react";
import cn from "classnames";
import memoizeOne from "memoize-one";

// src/compute-lines.ts
import * as diff from "diff";
var jsDiff = diff;
var DiffMethod = /* @__PURE__ */ ((DiffMethod2) => {
  DiffMethod2["CHARS"] = "diffChars";
  DiffMethod2["WORDS"] = "diffWords";
  DiffMethod2["WORDS_WITH_SPACE"] = "diffWordsWithSpace";
  DiffMethod2["LINES"] = "diffLines";
  DiffMethod2["TRIMMED_LINES"] = "diffTrimmedLines";
  DiffMethod2["SENTENCES"] = "diffSentences";
  DiffMethod2["CSS"] = "diffCss";
  return DiffMethod2;
})(DiffMethod || {});
var constructLines = (value) => {
  const lines = value.split("\n");
  const isAllEmpty = lines.every((val) => !val);
  if (isAllEmpty) {
    if (lines.length === 2) {
      return [];
    }
    lines.pop();
    return lines;
  }
  const lastLine = lines[lines.length - 1];
  const firstLine = lines[0];
  if (!lastLine) {
    lines.pop();
  }
  if (!firstLine) {
    lines.shift();
  }
  return lines;
};
var computeDiff = (oldValue, newValue, compareMethod = "diffChars" /* CHARS */) => {
  const diffArray = jsDiff[compareMethod](oldValue, newValue);
  const computedDiff = {
    left: [],
    right: []
  };
  diffArray.forEach(({ added, removed, value }) => {
    const diffInformation = {};
    if (added) {
      diffInformation.type = 1 /* ADDED */;
      diffInformation.value = value;
      computedDiff.right.push(diffInformation);
    }
    if (removed) {
      diffInformation.type = 2 /* REMOVED */;
      diffInformation.value = value;
      computedDiff.left.push(diffInformation);
    }
    if (!removed && !added) {
      diffInformation.type = 0 /* DEFAULT */;
      diffInformation.value = value;
      computedDiff.right.push(diffInformation);
      computedDiff.left.push(diffInformation);
    }
    return diffInformation;
  });
  return computedDiff;
};
var computeLineInformation = (oldString, newString, disableWordDiff = false, compareMethod = "diffChars" /* CHARS */, linesOffset = 0) => {
  const diffArray = diff.diffLines(oldString.trimRight(), newString.trimRight(), {
    newlineIsToken: true,
    ignoreWhitespace: false,
    ignoreCase: false
  });
  let rightLineNumber = linesOffset;
  let leftLineNumber = linesOffset;
  let lineInformation = [];
  let counter = 0;
  const diffLines2 = [];
  const ignoreDiffIndexes = [];
  const getLineInformation = (value, diffIndex, added, removed, evaluateOnlyFirstLine) => {
    const lines = constructLines(value);
    return lines.map((line, lineIndex) => {
      const left = {};
      const right = {};
      if (ignoreDiffIndexes.includes(`${diffIndex}-${lineIndex}`) || evaluateOnlyFirstLine && lineIndex !== 0) {
        return void 0;
      }
      if (added || removed) {
        if (!diffLines2.includes(counter)) {
          diffLines2.push(counter);
        }
        if (removed) {
          leftLineNumber += 1;
          left.lineNumber = leftLineNumber;
          left.type = 2 /* REMOVED */;
          left.value = line || " ";
          const nextDiff = diffArray[diffIndex + 1];
          if (nextDiff && nextDiff.added) {
            const nextDiffLines = constructLines(nextDiff.value)[lineIndex];
            if (nextDiffLines) {
              const {
                value: rightValue,
                lineNumber,
                type
              } = getLineInformation(nextDiff.value, diffIndex, true, false, true)[0].right;
              ignoreDiffIndexes.push(`${diffIndex + 1}-${lineIndex}`);
              right.lineNumber = lineNumber;
              right.type = type;
              if (disableWordDiff) {
                right.value = rightValue;
              } else {
                const computedDiff = computeDiff(line, rightValue, compareMethod);
                right.value = computedDiff.right;
                left.value = computedDiff.left;
              }
            }
          }
        } else {
          rightLineNumber += 1;
          right.lineNumber = rightLineNumber;
          right.type = 1 /* ADDED */;
          right.value = line;
        }
      } else {
        leftLineNumber += 1;
        rightLineNumber += 1;
        left.lineNumber = leftLineNumber;
        left.type = 0 /* DEFAULT */;
        left.value = line;
        right.lineNumber = rightLineNumber;
        right.type = 0 /* DEFAULT */;
        right.value = line;
      }
      counter += 1;
      return { right, left };
    }).filter(Boolean);
  };
  diffArray.forEach(({ added, removed, value }, index) => {
    lineInformation = [
      ...lineInformation,
      ...getLineInformation(value, index, added, removed)
    ];
  });
  return {
    lineInformation,
    diffLines: diffLines2
  };
};

// src/styles.ts
import { css, cx } from "@emotion/css";
var styles_default = (styleOverride, useDarkTheme = false) => {
  const { variables: overrideVariables = {}, ...styles } = styleOverride;
  const themeVariables = {
    light: {
      ...{
        diffViewerBackground: "#fff",
        diffViewerColor: "#212529",
        addedBackground: "#e6ffed",
        addedColor: "#24292e",
        removedBackground: "#ffeef0",
        removedColor: "#24292e",
        wordAddedBackground: "#acf2bd",
        wordRemovedBackground: "#fdb8c0",
        addedGutterBackground: "#cdffd8",
        removedGutterBackground: "#ffdce0",
        gutterBackground: "#f7f7f7",
        gutterBackgroundDark: "#f3f1f1",
        highlightBackground: "#fffbdd",
        highlightGutterBackground: "#fff5b1",
        codeFoldGutterBackground: "#dbedff",
        codeFoldBackground: "#f1f8ff",
        emptyLineBackground: "#fafbfc",
        gutterColor: "#212529",
        addedGutterColor: "#212529",
        removedGutterColor: "#212529",
        codeFoldContentColor: "#212529",
        diffViewerTitleBackground: "#fafbfc",
        diffViewerTitleColor: "#212529",
        diffViewerTitleBorderColor: "#eee"
      },
      ...overrideVariables.light || {}
    },
    dark: {
      ...{
        diffViewerBackground: "#2e303c",
        diffViewerColor: "#FFF",
        addedBackground: "#044B53",
        addedColor: "white",
        removedBackground: "#632F34",
        removedColor: "white",
        wordAddedBackground: "#055d67",
        wordRemovedBackground: "#7d383f",
        addedGutterBackground: "#034148",
        removedGutterBackground: "#632b30",
        gutterBackground: "#2c2f3a",
        gutterBackgroundDark: "#262933",
        highlightBackground: "#2a3967",
        highlightGutterBackground: "#2d4077",
        codeFoldGutterBackground: "#21232b",
        codeFoldBackground: "#262831",
        emptyLineBackground: "#363946",
        gutterColor: "#464c67",
        addedGutterColor: "#8c8c8c",
        removedGutterColor: "#8c8c8c",
        codeFoldContentColor: "#555a7b",
        diffViewerTitleBackground: "#2f323e",
        diffViewerTitleColor: "#555a7b",
        diffViewerTitleBorderColor: "#353846"
      },
      ...overrideVariables.dark || {}
    }
  };
  const variables = useDarkTheme ? themeVariables.dark : themeVariables.light;
  const content = css({
    width: "100%",
    label: "content"
  });
  const splitView = css({
    [`.${content}`]: {
      width: "50%"
    },
    label: "split-view"
  });
  const diffContainer = css({
    width: "100%",
    background: variables.diffViewerBackground,
    pre: {
      margin: 0,
      whiteSpace: "pre-wrap",
      lineHeight: "25px"
    },
    label: "diff-container",
    borderCollapse: "collapse"
  });
  const codeFoldContent = css({
    color: variables.codeFoldContentColor,
    label: "code-fold-content"
  });
  const contentText = css({
    color: variables.diffViewerColor,
    label: "content-text"
  });
  const titleBlock = css({
    background: variables.diffViewerTitleBackground,
    padding: 10,
    borderBottom: `1px solid ${variables.diffViewerTitleBorderColor}`,
    label: "title-block",
    ":last-child": {
      borderLeft: `1px solid ${variables.diffViewerTitleBorderColor}`
    },
    [`.${contentText}`]: {
      color: variables.diffViewerTitleColor
    }
  });
  const lineNumber = css({
    color: variables.gutterColor,
    label: "line-number"
  });
  const diffRemoved = css({
    background: variables.removedBackground,
    color: variables.removedColor,
    pre: {
      color: variables.removedColor
    },
    [`.${lineNumber}`]: {
      color: variables.removedGutterColor
    },
    label: "diff-removed"
  });
  const diffAdded = css({
    background: variables.addedBackground,
    color: variables.addedColor,
    pre: {
      color: variables.addedColor
    },
    [`.${lineNumber}`]: {
      color: variables.addedGutterColor
    },
    label: "diff-added"
  });
  const wordDiff = css({
    padding: 2,
    display: "inline-flex",
    borderRadius: 1,
    label: "word-diff"
  });
  const wordAdded = css({
    background: variables.wordAddedBackground,
    label: "word-added"
  });
  const wordRemoved = css({
    background: variables.wordRemovedBackground,
    label: "word-removed"
  });
  const codeFoldGutter = css({
    backgroundColor: variables.codeFoldGutterBackground,
    label: "code-fold-gutter"
  });
  const codeFold = css({
    backgroundColor: variables.codeFoldBackground,
    height: 40,
    fontSize: 14,
    fontWeight: 700,
    label: "code-fold",
    a: {
      textDecoration: "underline !important",
      cursor: "pointer",
      pre: {
        display: "inline"
      }
    }
  });
  const emptyLine = css({
    backgroundColor: variables.emptyLineBackground,
    label: "empty-line"
  });
  const marker = css({
    width: 25,
    paddingLeft: 10,
    paddingRight: 10,
    userSelect: "none",
    label: "marker",
    [`&.${diffAdded}`]: {
      pre: {
        color: variables.addedColor
      }
    },
    [`&.${diffRemoved}`]: {
      pre: {
        color: variables.removedColor
      }
    }
  });
  const highlightedLine = css({
    background: variables.highlightBackground,
    label: "highlighted-line",
    [`.${wordAdded}, .${wordRemoved}`]: {
      backgroundColor: "initial"
    }
  });
  const highlightedGutter = css({
    label: "highlighted-gutter"
  });
  const gutter = css({
    userSelect: "none",
    minWidth: 50,
    padding: "0 10px",
    label: "gutter",
    textAlign: "right",
    background: variables.gutterBackground,
    "&:hover": {
      cursor: "pointer",
      background: variables.gutterBackgroundDark,
      pre: {
        opacity: 1
      }
    },
    pre: {
      opacity: 0.5
    },
    [`&.${diffAdded}`]: {
      background: variables.addedGutterBackground
    },
    [`&.${diffRemoved}`]: {
      background: variables.removedGutterBackground
    },
    [`&.${highlightedGutter}`]: {
      background: variables.highlightGutterBackground,
      "&:hover": {
        background: variables.highlightGutterBackground
      }
    }
  });
  const emptyGutter = css({
    "&:hover": {
      background: variables.gutterBackground,
      cursor: "initial"
    },
    label: "empty-gutter"
  });
  const line = css({
    verticalAlign: "baseline",
    label: "line"
  });
  const defaultStyles = {
    diffContainer,
    diffRemoved,
    diffAdded,
    splitView,
    marker,
    highlightedGutter,
    highlightedLine,
    gutter,
    line,
    wordDiff,
    wordAdded,
    wordRemoved,
    codeFoldGutter,
    codeFold,
    emptyGutter,
    emptyLine,
    lineNumber,
    contentText,
    content,
    codeFoldContent,
    titleBlock
  };
  const computerOverrideStyles = Object.keys(styles).reduce((acc, key) => ({
    ...acc,
    ...{
      [key]: css(styles[key])
    }
  }), {});
  return Object.keys(defaultStyles).reduce((acc, key) => ({
    ...acc,
    ...{
      [key]: computerOverrideStyles[key] ? cx(defaultStyles[key], computerOverrideStyles[key]) : defaultStyles[key]
    }
  }), {});
};

// src/index.tsx
var LineNumberPrefix = /* @__PURE__ */ ((LineNumberPrefix2) => {
  LineNumberPrefix2["LEFT"] = "L";
  LineNumberPrefix2["RIGHT"] = "R";
  return LineNumberPrefix2;
})(LineNumberPrefix || {});
var DiffViewer = class extends React.Component {
  constructor(props) {
    super(props);
    this.resetCodeBlocks = () => {
      if (this.state.expandedBlocks.length > 0) {
        this.setState({
          expandedBlocks: []
        });
        return true;
      }
      return false;
    };
    this.onBlockExpand = (id) => {
      const prevState = this.state.expandedBlocks.slice();
      prevState.push(id);
      this.setState({
        expandedBlocks: prevState
      });
    };
    this.computeStyles = memoizeOne(styles_default);
    this.onLineNumberClickProxy = (id) => {
      if (this.props.onLineNumberClick) {
        return (e) => this.props.onLineNumberClick(id, e);
      }
      return () => {
      };
    };
    this.renderWordDiff = (diffArray, renderer) => {
      return diffArray.map((wordDiff, i) => {
        return /* @__PURE__ */ React.createElement("span", {
          key: i,
          className: cn(this.styles.wordDiff, {
            [this.styles.wordAdded]: wordDiff.type === 1 /* ADDED */,
            [this.styles.wordRemoved]: wordDiff.type === 2 /* REMOVED */
          })
        }, renderer ? renderer(wordDiff.value) : wordDiff.value);
      });
    };
    this.renderLine = (lineNumber, type, prefix, value, additionalLineNumber, additionalPrefix) => {
      const lineNumberTemplate = `${prefix}-${lineNumber}`;
      const additionalLineNumberTemplate = `${additionalPrefix}-${additionalLineNumber}`;
      const highlightLine = this.props.highlightLines.includes(lineNumberTemplate) || this.props.highlightLines.includes(additionalLineNumberTemplate);
      const added = type === 1 /* ADDED */;
      const removed = type === 2 /* REMOVED */;
      let content;
      if (Array.isArray(value)) {
        content = this.renderWordDiff(value, this.props.renderContent);
      } else if (this.props.renderContent) {
        content = this.props.renderContent(value);
      } else {
        content = value;
      }
      return /* @__PURE__ */ React.createElement(React.Fragment, null, !this.props.hideLineNumbers && /* @__PURE__ */ React.createElement("td", {
        onClick: lineNumber && this.onLineNumberClickProxy(lineNumberTemplate),
        className: cn(this.styles.gutter, {
          [this.styles.emptyGutter]: !lineNumber,
          [this.styles.diffAdded]: added,
          [this.styles.diffRemoved]: removed,
          [this.styles.highlightedGutter]: highlightLine
        })
      }, /* @__PURE__ */ React.createElement("pre", {
        className: this.styles.lineNumber
      }, lineNumber)), !this.props.splitView && !this.props.hideLineNumbers && /* @__PURE__ */ React.createElement("td", {
        onClick: additionalLineNumber && this.onLineNumberClickProxy(additionalLineNumberTemplate),
        className: cn(this.styles.gutter, {
          [this.styles.emptyGutter]: !additionalLineNumber,
          [this.styles.diffAdded]: added,
          [this.styles.diffRemoved]: removed,
          [this.styles.highlightedGutter]: highlightLine
        })
      }, /* @__PURE__ */ React.createElement("pre", {
        className: this.styles.lineNumber
      }, additionalLineNumber)), /* @__PURE__ */ React.createElement("td", {
        className: cn(this.styles.marker, {
          [this.styles.emptyLine]: !content,
          [this.styles.diffAdded]: added,
          [this.styles.diffRemoved]: removed,
          [this.styles.highlightedLine]: highlightLine
        })
      }, /* @__PURE__ */ React.createElement("pre", null, added && "+", removed && "-")), /* @__PURE__ */ React.createElement("td", {
        className: cn(this.styles.content, {
          [this.styles.emptyLine]: !content,
          [this.styles.diffAdded]: added,
          [this.styles.diffRemoved]: removed,
          [this.styles.highlightedLine]: highlightLine
        })
      }, /* @__PURE__ */ React.createElement("pre", {
        className: this.styles.contentText
      }, content)));
    };
    this.renderSplitView = ({ left, right }, index) => {
      return /* @__PURE__ */ React.createElement("tr", {
        key: index,
        className: this.styles.line
      }, this.renderLine(left.lineNumber, left.type, "L" /* LEFT */, left.value), this.renderLine(right.lineNumber, right.type, "R" /* RIGHT */, right.value));
    };
    this.renderInlineView = ({ left, right }, index) => {
      let content;
      if (left.type === 2 /* REMOVED */ && right.type === 1 /* ADDED */) {
        return /* @__PURE__ */ React.createElement(React.Fragment, {
          key: index
        }, /* @__PURE__ */ React.createElement("tr", {
          className: this.styles.line
        }, this.renderLine(left.lineNumber, left.type, "L" /* LEFT */, left.value, null)), /* @__PURE__ */ React.createElement("tr", {
          className: this.styles.line
        }, this.renderLine(null, right.type, "R" /* RIGHT */, right.value, right.lineNumber)));
      }
      if (left.type === 2 /* REMOVED */) {
        content = this.renderLine(left.lineNumber, left.type, "L" /* LEFT */, left.value, null);
      }
      if (left.type === 0 /* DEFAULT */) {
        content = this.renderLine(left.lineNumber, left.type, "L" /* LEFT */, left.value, right.lineNumber, "R" /* RIGHT */);
      }
      if (right.type === 1 /* ADDED */) {
        content = this.renderLine(null, right.type, "R" /* RIGHT */, right.value, right.lineNumber);
      }
      return /* @__PURE__ */ React.createElement("tr", {
        key: index,
        className: this.styles.line
      }, content);
    };
    this.onBlockClickProxy = (id) => () => this.onBlockExpand(id);
    this.renderSkippedLineIndicator = (num, blockNumber, leftBlockLineNumber, rightBlockLineNumber) => {
      const { hideLineNumbers, splitView } = this.props;
      const message = this.props.codeFoldMessageRenderer ? this.props.codeFoldMessageRenderer(num, leftBlockLineNumber, rightBlockLineNumber) : /* @__PURE__ */ React.createElement("pre", {
        className: this.styles.codeFoldContent
      }, "Expand ", num, " lines ...");
      const content = /* @__PURE__ */ React.createElement("td", null, /* @__PURE__ */ React.createElement("a", {
        onClick: this.onBlockClickProxy(blockNumber),
        tabIndex: 0
      }, message));
      const isUnifiedViewWithoutLineNumbers = !splitView && !hideLineNumbers;
      return /* @__PURE__ */ React.createElement("tr", {
        key: `${leftBlockLineNumber}-${rightBlockLineNumber}`,
        className: this.styles.codeFold
      }, !hideLineNumbers && /* @__PURE__ */ React.createElement("td", {
        className: this.styles.codeFoldGutter
      }), /* @__PURE__ */ React.createElement("td", {
        className: cn({
          [this.styles.codeFoldGutter]: isUnifiedViewWithoutLineNumbers
        })
      }), isUnifiedViewWithoutLineNumbers ? /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement("td", null), content) : /* @__PURE__ */ React.createElement(React.Fragment, null, content, /* @__PURE__ */ React.createElement("td", null)), /* @__PURE__ */ React.createElement("td", null), /* @__PURE__ */ React.createElement("td", null));
    };
    this.renderDiff = () => {
      const {
        oldValue,
        newValue,
        splitView,
        disableWordDiff,
        compareMethod,
        linesOffset
      } = this.props;
      const { lineInformation, diffLines: diffLines2 } = computeLineInformation(oldValue, newValue, disableWordDiff, compareMethod, linesOffset);
      const extraLines = this.props.extraLinesSurroundingDiff < 0 ? 0 : this.props.extraLinesSurroundingDiff;
      let skippedLines = [];
      return lineInformation.map((line, i) => {
        const diffBlockStart = diffLines2[0];
        const currentPosition = diffBlockStart - i;
        if (this.props.showDiffOnly) {
          if (currentPosition === -extraLines) {
            skippedLines = [];
            diffLines2.shift();
          }
          if (line.left.type === 0 /* DEFAULT */ && (currentPosition > extraLines || typeof diffBlockStart === "undefined") && !this.state.expandedBlocks.includes(diffBlockStart)) {
            skippedLines.push(i + 1);
            if (i === lineInformation.length - 1 && skippedLines.length > 1) {
              return this.renderSkippedLineIndicator(skippedLines.length, diffBlockStart, line.left.lineNumber, line.right.lineNumber);
            }
            return null;
          }
        }
        const diffNodes = splitView ? this.renderSplitView(line, i) : this.renderInlineView(line, i);
        if (currentPosition === extraLines && skippedLines.length > 0) {
          const { length } = skippedLines;
          skippedLines = [];
          return /* @__PURE__ */ React.createElement(React.Fragment, {
            key: i
          }, this.renderSkippedLineIndicator(length, diffBlockStart, line.left.lineNumber, line.right.lineNumber), diffNodes);
        }
        return diffNodes;
      });
    };
    this.render = () => {
      const {
        oldValue,
        newValue,
        useDarkTheme,
        leftTitle,
        rightTitle,
        splitView,
        hideLineNumbers
      } = this.props;
      if (typeof oldValue !== "string" || typeof newValue !== "string") {
        throw Error('"oldValue" and "newValue" should be strings');
      }
      this.styles = this.computeStyles(this.props.styles, useDarkTheme);
      const nodes = this.renderDiff();
      const colSpanOnSplitView = hideLineNumbers ? 2 : 3;
      const colSpanOnInlineView = hideLineNumbers ? 2 : 4;
      const title = (leftTitle || rightTitle) && /* @__PURE__ */ React.createElement("tr", null, /* @__PURE__ */ React.createElement("td", {
        colSpan: splitView ? colSpanOnSplitView : colSpanOnInlineView,
        className: this.styles.titleBlock
      }, /* @__PURE__ */ React.createElement("pre", {
        className: this.styles.contentText
      }, leftTitle)), splitView && /* @__PURE__ */ React.createElement("td", {
        colSpan: colSpanOnSplitView,
        className: this.styles.titleBlock
      }, /* @__PURE__ */ React.createElement("pre", {
        className: this.styles.contentText
      }, rightTitle)));
      return /* @__PURE__ */ React.createElement("table", {
        className: cn(this.styles.diffContainer, {
          [this.styles.splitView]: splitView
        })
      }, /* @__PURE__ */ React.createElement("tbody", null, title, nodes));
    };
    this.state = {
      expandedBlocks: []
    };
  }
};
DiffViewer.defaultProps = {
  oldValue: "",
  newValue: "",
  splitView: true,
  highlightLines: [],
  disableWordDiff: false,
  compareMethod: "diffChars" /* CHARS */,
  styles: {},
  hideLineNumbers: false,
  extraLinesSurroundingDiff: 3,
  showDiffOnly: true,
  useDarkTheme: false,
  linesOffset: 0
};
var src_default = DiffViewer;
export {
  DiffMethod,
  LineNumberPrefix,
  src_default as default
};
//# sourceMappingURL=index.mjs.map